"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.FileDataCache = exports.CACHE_CHECK_INTERVAL = void 0;
const path_1 = require("path");
const fs_1 = require("fs");
exports.CACHE_CHECK_INTERVAL = 20000;
class FileDataCache {
    constructor(options) {
        var _a;
        this.map = new Map();
        this.loadFileData = options.loadFileData;
        this.checkInterval = (_a = options.checkInterval) !== null && _a !== void 0 ? _a : exports.CACHE_CHECK_INTERVAL;
        this.readFile = Boolean(options.readFile);
    }
    getEntries() {
        return Array.from(this.map.entries()).map(([path, data]) => (Object.assign(Object.assign({}, data), { path })));
    }
    getPaths() {
        return Array.from(this.map.keys());
    }
    getValues() {
        return Array.from(this.map.values()).map(({ value }) => value);
    }
    getEntry(fullPath) {
        return this.map.get(fullPath);
    }
    loadData(filePath, options = {}) {
        let hasChanged = false;
        const now = Date.now();
        const cachedEntry = this.getEntry(filePath);
        const { bypassCache = false } = options;
        if (cachedEntry && !bypassCache) {
            if (now - cachedEntry.lastCheck.time < this.checkInterval) {
                // The file was recently checked: return cached values
                return cachedEntry.value;
            }
        }
        const fileExists = fs_1.existsSync(filePath);
        let lastModified = 0;
        let value;
        const filename = path_1.basename(filePath);
        if (fileExists) {
            // Check the last modified date of the file
            lastModified = getFileLastModifiedDate(filePath);
            if (cachedEntry && lastModified === cachedEntry.lastModified) {
                // No change
                value = cachedEntry.value;
            }
            else {
                // Changes detected.
                try {
                    const fileContent = this.readFile ? readFile(filePath) : undefined;
                    value = this.loadFileData(filePath, fileContent);
                    hasChanged = true;
                }
                catch (e) {
                    console.warn(`Fail to load file: ${filename}`, e);
                }
            }
            if (!(cachedEntry === null || cachedEntry === void 0 ? void 0 : cachedEntry.fileExists)) {
                hasChanged = true;
            }
        }
        else {
            console.warn('Path not found: ', filePath);
            if (cachedEntry === null || cachedEntry === void 0 ? void 0 : cachedEntry.fileExists) {
                hasChanged = true;
                value = undefined;
            }
        }
        const cachedFileData = {
            lastCheck: {
                time: now,
                wasChanged: hasChanged,
            },
            fileExists,
            lastModified,
            value,
        };
        this.map.set(filePath, cachedFileData);
        return value;
    }
}
exports.FileDataCache = FileDataCache;
function readFile(filePath) {
    let fileContent;
    try {
        fileContent = fs_1.readFileSync(filePath, 'utf8');
    }
    catch (e) { }
    return fileContent;
}
function getFileLastModifiedDate(path) {
    try {
        const stats = fs_1.statSync(path);
        return stats.mtimeMs;
    }
    catch (e) {
        return null;
    }
}
//# sourceMappingURL=FileDataCache.js.map